<?php

declare(strict_types=1);

use JetBrains\PhpStorm\NoReturn;
use Lidonation\Adapayments\App\Enums\CurrencyEnum;
use Lidonation\Adapayments\App\Services\AdaUsdQuoteService;

if (! function_exists('ada_to_usd')) {
    function ada_to_usd(int|float $amount = null): float|int
    {
        $rateInstance = new AdaUsdQuoteService(CurrencyEnum::ADA, CurrencyEnum::USD);

        return isset($amount) ? $rateInstance->rate * $amount : $rateInstance->rate;
    }
}

if (! function_exists('usd_to_ada')) {
    function usd_to_ada(int|float $amount = null): float|int
    {
        $rateInstance = new AdaUsdQuoteService(CurrencyEnum::USD, CurrencyEnum::ADA);

        return isset($amount) ? $rateInstance->rate * $amount : $rateInstance->rate;
    }
}

if (! function_exists('env')) {
    /**
     * Gets the value of an environment variable. Supports boolean, empty and null.
     *
     * @param string $key
     * @param mixed|null $default
     * @return mixed
     */
    function env(string $key, mixed $default = null): mixed
    {
        $value = $_ENV[$key] ?? null;


        if (!$value) {
            return value($default);
        }

        switch (strtolower($value)) {
            case 'true':
            case '(true)':
                return true;

            case 'false':
            case '(false)':
                return false;

            case 'empty':
            case '(empty)':
                return '';

            case 'null':
            case '(null)':
                return null;
        }

        if ( str_starts_with($value, '"') && str_ends_with($value, '"')) {
            return substr($value, 1, -1);
        }

        return $value;
    }
}

if (! function_exists('app_base_path')) {
    function app_base_path($path = null): string
    {
        return !!$path 
            ? __DIR__.  '/../' . $path
            : __DIR__.  '/../';
    }
}

if (! function_exists('_view')) {
    function _view($path, $attributes = []): void
    {
        extract($attributes);

        require app_base_path('src/resources/views/' . $path);
    }
}

if (! function_exists('redirect')) {
    #[NoReturn] function redirect($path): void
    {
        header("location: {$path}");
        exit();
    }
}

if (! function_exists('abort')) {
    #[NoReturn] function abort(): void
    {
        require app_base_path("src/resources/views/abort.php");

        die();
    }
}
