"use strict";

import View from './View.js';
import { Lucid, Blockfrost, Tx} from "https://unpkg.com/lucid-cardano@0.10.7/web/mod.js"

class connectWalletView extends View {
    parentElement = document.querySelector('#connectWallet');
    button = this.parentElement.querySelector('#connectWallet-button');
    optionsSection = document.querySelector('.connectWallet-options');
    walletCheckboxes = document.querySelectorAll("input[name=wallet-checkbox]");
    walltConnected = false;
    blockfrost;
    api;
    lucid;

    persistWallet(walletName) {
        localStorage.setItem('walletName', walletName);
    }

    renderWalletConnectingSpinner(walletName) {
        const markup = `
            <button
                class="caret-white inline-flex w-full justify-center gap-x-1.5 rounded-md bg-white px-3 py-2 text-sm font-semibold text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300 hover:bg-gray-50">
                <svg
                    class="relative w-2 h-2 lg:h-4 lg:w-4 border-black border-t-2 border-b-2 rounded-full animate-spin"
                    viewBox="0 0 24 24">
                </svg>
                connecting ${walletName}
                <span id="cancel-connection"class="cursor-pointer hover:bg-red">x</span>
            </button>
        `;
    this.parentElement.innerHTML =  markup;
    }

    renderWalletConnect(walletName) {
        const markup = `
            <div
                class="caret-white inline-flex w-full justify-center gap-x-1.5 rounded-md bg-white px-3 py-2 text-sm font-semibold text-gray-900 shadow-sm ring-1 ring-inset ring-gray-300">
                <span class="p-1 pointer-events-none">${walletName} connected</span>
                <span id="disconnect-wallet"class="p-1 cursor-pointer hover:bg-red">x</span>
            </div>
        `;
        this.parentElement.innerHTML =  markup;
    }

    disableCheckboxes() {
        this.walletCheckboxes.forEach(function (checkbox) {
            checkbox.checked = false;
        });
    };

    addHandlerRender(handler, blockfrost) {
        this.blockfrost = blockfrost;

        this.button.addEventListener('click', function (e) {
            e.preventDefault();
            document.querySelector('#connectWallet-options').classList.toggle('hidden');
        })

        handler();
    };

    isSupported(wallet) {
        if (typeof window.cardano === 'undefined') {
            return false;
        }
        return (!!window.cardano[wallet]) && typeof window.cardano[wallet] != 'undefined';
    };

    async enableWallet(wallet) {

        if (
            typeof window.cardano === "undefined" ||
            !window?.cardano ||
            !window.cardano[wallet]
        ) {
            return false;
        }

        console.log(`${wallet} enabled.`);
        return window.cardano[wallet]?.enable();
    }

    async connectWallet(wallet) {
        try {
            if (!this.lucid || typeof this.lucid === "undefined") {
                return await this.init(wallet);
            } else {
                const api = await this.enableWallet(wallet);
                this.lucid.selectWallet(api);
                this.api = api;

                this.walletConnected = true;
                return true;
            }
        } catch (e) {
            console.log({ e });
            return false;
        }
    }

    async init(wallet) {
        if (!!this.lucid || typeof this.lucid !== "undefined") {
            const api = await this.enableWallet(wallet);
            this.lucid.selectWallet(api);
            this.api = api;
            return true;
        }
        let lucid;

        try {
            const api = await this.enableWallet(wallet);

            if (!api) {
                console.log(`${wallet} not installed!`);
                return;
            }

            const blockfrostUrl = this.blockfrost.url;
            const blockfrostKey = this.blockfrost.projectId;
            const networkId = await  api.getNetworkId();
            let network;
            switch (networkId) {
                case 0:
                    network = 'Preview';
                    break;
                default:
                    network = 'Mainnet';
            }
        
            lucid = await Lucid.new(
                new Blockfrost(blockfrostUrl, blockfrostKey),
                network,
              );

            lucid = lucid.selectWallet(api);
            this.lucid = lucid;
            this.api = api;
            
            this.walletConnected = true;
            return true
        } catch (e) {
            console.log(e);
            return false;
        }
    }

    async payToAddress(address, lovelaceAmount) {
        try {
            let assets = {lovelace: BigInt(lovelaceAmount)};
            let tx = await new Tx(this.lucid)
                .payToAddress(address, assets)
                .validTo(Date.now() + 1000000)
                .complete();
    
            const signedTx = await tx.sign().complete();
    
            const txHash = await signedTx.submit();
            
            return true;
        } catch (e) {
            console.log(e);
            return false;
        }

    }
}

export default new connectWalletView();
